#include "player_input.h"

#include "core/input/input_event.h"
#include "macros.h"

String const PlayerInput::signal_movement{ "movement" };
String const PlayerInput::signal_look{ "look" };
String const PlayerInput::signal_mouselook{ "mouselook" };

void PlayerInput::_bind_methods() {
	BIND_PROPERTY(Variant::STRING, action_move_left);
	BIND_PROPERTY(Variant::STRING, action_move_right);
	BIND_PROPERTY(Variant::STRING, action_move_forward);
	BIND_PROPERTY(Variant::STRING, action_move_back);
	BIND_PROPERTY(Variant::STRING, action_look_left);
	BIND_PROPERTY(Variant::STRING, action_look_right);
	BIND_PROPERTY(Variant::STRING, action_look_up);
	BIND_PROPERTY(Variant::STRING, action_look_down);
	ADD_SIGNAL(MethodInfo(self_type::signal_movement, PropertyInfo(Variant::VECTOR2, "movement_directions")));
	ADD_SIGNAL(MethodInfo(self_type::signal_look, PropertyInfo(Variant::VECTOR2, "look_delta")));
	ADD_SIGNAL(MethodInfo(self_type::signal_mouselook, PropertyInfo(Variant::VECTOR2, "mouse_delta")));
}

void PlayerInput::_notification(int what) {
	if (Engine::get_singleton()->is_editor_hint()) {
		return;
	}
	switch (what) {
		default:
			return;
		case NOTIFICATION_READY:
			set_process_unhandled_input(true);
			Input::get_singleton()->set_mouse_mode(Input::MOUSE_MODE_CAPTURED);
			return;
	}
}

void PlayerInput::unhandled_input(Ref<InputEvent> const &event) {
	Input *input{ Input::get_singleton() };

	bool is_movement_action{ event->is_action(self_type::action_move_left) };
	is_movement_action |= event->is_action(self_type::action_move_right);
	is_movement_action |= event->is_action(self_type::action_move_forward);
	is_movement_action |= event->is_action(self_type::action_move_back);
	if (is_movement_action) {
		float const h_axis{ input->get_axis(self_type::action_move_left, self_type::action_move_right) };
		float const v_axis{ input->get_axis(self_type::action_move_forward, self_type::action_move_back) };
		emit_signal(self_type::signal_movement, Vector2{ h_axis, v_axis });
	}

	bool is_look_action{ event->is_action(self_type::action_look_left) };
	is_look_action |= event->is_action(self_type::action_look_right);
	is_look_action |= event->is_action(self_type::action_look_up);
	is_look_action |= event->is_action(self_type::action_look_down);
	if (is_look_action) {
		float const h_axis{ input->get_axis(self_type::action_look_right, self_type::action_look_left) };
		float const v_axis{ input->get_axis(self_type::action_look_down, self_type::action_look_up) };
		emit_signal(self_type::signal_look, Vector2{ h_axis, v_axis } * this->look_speed);
	}

	Ref<InputEventMouseMotion> mouse_motion_action{ event };
	if (mouse_motion_action.is_valid()) {
		Vector2 const velocity{ mouse_motion_action->get_relative() };
		emit_signal(self_type::signal_mouselook, velocity * this->mouselook_speed);
	}
}

void PlayerInput::set_mouselook_speed(Vector2 speed) {
	this->mouselook_speed = speed;
}

Vector2 PlayerInput::get_mouselook_speed() const {
	return this->mouselook_speed;
}

void PlayerInput::set_look_speed(Vector2 speed) {
	this->look_speed = speed;
}

Vector2 PlayerInput::get_look_speed() const {
	return this->look_speed;
}

void PlayerInput::set_action_move_left(String action) {
	this->action_move_left = action;
}

String PlayerInput::get_action_move_left() const {
	return this->action_move_left;
}

void PlayerInput::set_action_move_right(String action) {
	this->action_move_right = action;
}

String PlayerInput::get_action_move_right() const {
	return this->action_move_right;
}

void PlayerInput::set_action_move_forward(String action) {
	this->action_move_forward = action;
}

String PlayerInput::get_action_move_forward() const {
	return this->action_move_forward;
}

void PlayerInput::set_action_move_back(String action) {
	this->action_move_back = action;
}

String PlayerInput::get_action_move_back() const {
	return this->action_move_back;
}

void PlayerInput::set_action_look_left(String action) {
	this->action_look_left = action;
}

String PlayerInput::get_action_look_left() const {
	return this->action_look_left;
}

void PlayerInput::set_action_look_right(String action) {
	this->action_look_right = action;
}

String PlayerInput::get_action_look_right() const {
	return this->action_look_right;
}

void PlayerInput::set_action_look_up(String action) {
	this->action_look_up = action;
}

String PlayerInput::get_action_look_up() const {
	return this->action_look_up;
}

void PlayerInput::set_action_look_down(String action) {
	this->action_look_down = action;
}

String PlayerInput::get_action_look_down() const {
	return this->action_look_down;
}
